@@ -125,6 +125,56 @@ gs_plugin_destroy (GsPlugin *plugin)
 	g_object_unref (plugin->priv->store);
 }
 
+#if AS_CHECK_VERSION(0,3,2)
+/**
+ * gs_plugin_appstream_get_origins_hash:
+ *
+ * Returns: A hash table with a string key of the application origin and a
+ * value of the guint percentage of the store is made up by that origin.
+ */
+static GHashTable *
+gs_plugin_appstream_get_origins_hash (GPtrArray *array)
+{
+	AsApp *app;
+	GHashTable *origins = NULL;
+	GList *keys = NULL;
+	GList *l;
+	const gchar *tmp;
+	gdouble perc;
+	guint *cnt;
+	guint i;
+
+	/* create a hash table with origin:cnt */
+	origins = g_hash_table_new_full (g_str_hash, g_str_equal,
+					 g_free, g_free);
+	for (i = 0; i < array->len; i++) {
+		app = g_ptr_array_index (array, i);
+		tmp = as_app_get_origin (app);
+		if (tmp == NULL)
+			continue;
+		cnt = g_hash_table_lookup (origins, tmp);
+		if (cnt == NULL) {
+			cnt = g_new0 (guint, 1);
+			g_hash_table_insert (origins, g_strdup (tmp), cnt);
+		}
+		(*cnt)++;
+	}
+
+	/* convert the cnt to a percentage */
+	keys = g_hash_table_get_keys (origins);
+	for (l = keys; l != NULL; l = l->next) {
+		tmp = l->data;
+		cnt = g_hash_table_lookup (origins, tmp);
+		perc = (100.f / (gdouble) array->len) * (gdouble) (*cnt);
+		g_debug ("origin %s provides %i apps (%.0f%%)", tmp, *cnt, perc);
+		*cnt = perc;
+	}
+
+	g_list_free (keys);
+	return origins;
+}
+#endif
+
 /**
  * gs_plugin_startup:
  */
@@ -137,6 +187,12 @@ gs_plugin_startup (GsPlugin *plugin, GError **error)
 #if AS_CHECK_VERSION(0,3,1)
 	guint i;
 #endif
+#if AS_CHECK_VERSION(0,3,2)
+	AsApp *app;
+	GHashTable *origins = NULL;
+	const gchar *origin;
+	guint *perc;
+#endif
 
 	/* clear all existing applications if the store was invalidated */
 	as_store_remove_all (plugin->priv->store);
@@ -174,10 +230,26 @@ gs_plugin_startup (GsPlugin *plugin, GError **error)
 		goto out;
 	}
 
+	/* add search terms for apps not in the main source */
+#if AS_CHECK_VERSION(0,3,2)
+	origins = gs_plugin_appstream_get_origins_hash (items);
+	for (i = 0; i < items->len; i++) {
+		app = g_ptr_array_index (items, i);
+		origin = as_app_get_origin (app);
+		if (origin == NULL)
+			continue;
+		perc = g_hash_table_lookup (origins, origin);
+		if (*perc < 10) {
+			g_debug ("Adding keyword '%s' to %s",
+				 origin, as_app_get_id (app));
+			as_app_add_keyword (app, NULL, origin, -1);
+		}
+	}
+#endif
+
 	/* look for any application with a HiDPI icon kudo */
 #if AS_CHECK_VERSION(0,3,1)
 	for (i = 0; i < items->len; i++) {
-		AsApp *app;
 		app = g_ptr_array_index (items, i);
 		if (as_app_has_kudo_kind (app, AS_KUDO_KIND_HI_DPI_ICON)) {
 			plugin->priv->has_hi_dpi_support = TRUE;
@@ -186,6 +258,10 @@ gs_plugin_startup (GsPlugin *plugin, GError **error)
 	}
 #endif
 out:
+#if AS_CHECK_VERSION(0,3,2)
+	if (origins != NULL)
+		g_hash_table_unref (origins);
+#endif
 	gs_profile_stop (plugin->profile, "appstream::startup");
 	return ret;
 }
@@ -208,6 +284,7 @@ gs_plugin_refine_item_pixbuf (GsPlugin *plugin, GsApp *app, AsApp *item)
 		gs_app_set_icon (app, as_icon_get_name (icon));
 		break;
 	case AS_ICON_KIND_STOCK:
+	case AS_ICON_KIND_LOCAL:
 		gs_app_set_icon (app, as_icon_get_name (icon));
 		ret = gs_app_load_icon (app, plugin->scale, &error);
 		if (!ret) {
@@ -309,6 +386,9 @@ gs_plugin_refine_item_pixbuf (GsPlugin *plugin, GsApp *app, AsApp *item)
 	case AS_ICON_KIND_REMOTE:
 		gs_app_set_icon (app, icon);
 		break;
+#if AS_CHECK_VERSION(0,2,7)
+	case AS_ICON_KIND_LOCAL:
+#endif
 	case AS_ICON_KIND_STOCK:
 		gs_app_set_icon (app, icon);
 		ret = gs_app_load_icon (app, plugin->scale, &error);
@@ -641,6 +721,7 @@ gs_plugin_refine_item (GsPlugin *plugin,
 	if (tmp != NULL) {
 		from_xml = as_markup_convert_simple (tmp, -1, error);
 		if (from_xml == NULL) {
+			g_prefix_error (error, "trying to parse '%s': ", tmp);
 			ret = FALSE;
 			goto out;
 		}
@@ -1192,9 +1273,9 @@ gs_plugin_add_popular_from_category (GsPlugin *plugin,
 				     GError **error)
 {
 	AsApp *item;
+	GError *error_local = NULL;
 	GPtrArray *array;
 	GsApp *app;
-	gboolean ret = TRUE;
 	guint i;
 
 	/* search categories for the search term */
@@ -1224,9 +1305,14 @@ gs_plugin_add_popular_from_category (GsPlugin *plugin,
 
 		/* add application */
 		app = gs_app_new (as_app_get_id_full (item));
-		ret = gs_plugin_refine_item (plugin, app, item, error);
-		if (!ret)
-			goto out;
+		if (!gs_plugin_refine_item (plugin, app, item, &error_local)) {
+			g_warning ("Failed to refine %s: %s",
+				   as_app_get_id (item),
+				   error_local->message);
+			g_clear_error (&error_local);
+			g_object_unref (app);
+			continue;
+		}
 
 		/* only suggest awesome applications */
 		if (gs_plugin_appstream_is_app_awesome (app)) {
@@ -1239,8 +1325,7 @@ gs_plugin_add_popular_from_category (GsPlugin *plugin,
 		}
 		g_object_unref (app);
 	}
-out:
-	return ret;
+	return TRUE;
 }
 
 /**
